/*
  Mex file to match patterns

  The calling syntax is:

     iprod = match_pat(swt,pattern,gamma,delays,mode);

  See match_patterns.m for more information
  
  Copyright (c) 1998 Thomas T. Liu


*/
#include <math.h>
#include "mex.h"

#define DOUBLE double
#define LONG long

#define	max(A, B)	((A) > (B) ? (A) : (B))
#define	min(A, B)	((A) < (B) ? (A) : (B))
/* Input Arguments */

#define	SWT	prhs[0]
#define	PATTERN	prhs[1]
#define GAMMA   prhs[2]
#define DELAYS  prhs[3]
#define MODE    prhs[4]

/* Output Arguments */

#define	IPROD	plhs[0]





#ifdef __STDC__
void mexFunction(
	int		nlhs,
	Matrix	*plhs[],
	int		nrhs,
	Matrix	*prhs[]
	)
#else
mexFunction(nlhs, plhs, nrhs, prhs)
int nlhs, nrhs;
Matrix *plhs[], *prhs[];
#endif
{
        DOUBLE *swt, *pattern,*iprodout,*delays;
        DOUBLE gamma;

        int dmin,dmax,numdelays,mode;
        unsigned int nr,nc,M,N,npat;

	/* Check for proper number of arguments */

	if (nrhs < 4) {
		mexErrMsgTxt("match_pat requires at least 4 input arguments.");
	}
 
        if( nrhs == 5){
	  mode = mxGetPr(MODE)[0];
	}else{
	  mode = 0;
	}

	if(mode == 0){
         numdelays = max(mxGetM(DELAYS),mxGetN(DELAYS));
         if(numdelays < 2){
	   mexErrMsgTxt("delays must have at least 2 elements when mode = 0.");
	 }  else{
	   dmin = mxGetPr(DELAYS)[0];
	   dmax = mxGetPr(DELAYS)[1];
	   numdelays = dmax-dmin+1;
	 }
	}else{
	  numdelays = max(mxGetM(DELAYS),mxGetN(DELAYS));
	}


	nr = mxGetM(SWT);
        nc = mxGetN(SWT);
        N = nr;
        M = nc-1;

        npat = mxGetM(PATTERN);


	swt = mxGetPr(SWT);
        pattern = mxGetPr(PATTERN);
        gamma = mxGetPr(GAMMA)[0];


        IPROD = mxCreateFull((unsigned int) numdelays, 1, REAL);
        iprodout = mxGetPr(IPROD);
	delays =  mxGetPr(DELAYS);



                
	matchcore(iprodout,swt,pattern,gamma,N,M,npat,delays,numdelays,mode);


}

matchcore(iprodout,swt,pattern,gamma,N,M,npat,delays,numdelays,mode)
DOUBLE iprodout[],swt[],pattern[],delays[];
DOUBLE gamma;
unsigned int N,M,npat;
int mode,numdelays;
{
  DOUBLE *pcoeff;
  int *pscales,*pindices;
  int i,j,index,delay;


   pcoeff = (DOUBLE *) malloc(npat*sizeof(DOUBLE));
   pscales = (int *) malloc(npat*sizeof(int));
   pindices = (int *) malloc(npat*sizeof(int));


  for (i =  0;i < npat;i++){
    pscales[i] = (int) pattern[i];
    pindices[i] =(int)  pattern[npat + i];
    pcoeff[i] = pattern[2*npat+i]*exp(-gamma*pattern[i]*log(2));
  }

  for(i =0;i < npat;i++){
    if (pscales[i] == (M+1))
      pscales[i] = 0;
  }

  /*
  printf("npat = %d\n",npat);
  for (i =  0;i < npat;i++){
    printf("%d %d %f\n",pscales[i],pindices[i],pcoeff[i]);
  }
  */



  if(mode == 0){
    i = 0;
    for(delay = (int) delays[0];delay <= (int) delays[1];delay++){

      iprodout[i] = 0;
      for(j = 0;j < npat;j++){
	index = (((pindices[j]+delay-1)%N)+N )%N;
	iprodout[i] = iprodout[i] + swt[N*pscales[j]+index]*pcoeff[j];
      }
      ++i;
    }
  }else{
    for(i = 0;i < numdelays;i++){
      iprodout[i] = 0;
      for(j = 0;j < npat;j++){
	index = (((pindices[j]+(int) delays[i]-1)%N)+N )%N;
	iprodout[i] = iprodout[i] + swt[N*pscales[j]+index]*pcoeff[j];
      }
    }
  }
}
