function iprod = match_patterns(swt,pattern,gamma,delays,mode);

% match_patterns -- compute inner product of a swt with a 
%                   signal pattern for a range of delays
%
% Usage
%  iprod = match_patterns(swt,pattern,gamma,delays);
%
% Inputs
%  swt		stationary wavelet transform table
%  pattern	signal pattern used for matching 
%  gamma	1/f^gamma parameter
%  delays	delay vector (see mode)
%  mode         0: (default) interpret delays as vector of delay endpoints
%		    e.g. [mindelay maxdelay]
%               1: interpret delays as a vector of delays
%		   e.g. [d0 d1 d2 ....dN]
%
% Outputs
%  iprod	inner product as a function of delay
%
% See Also
%  dstat_pattern

	if mode == 0
 	 if (length(delays) < 2)
		error('delays vec must have at least 2 elements when mode = 0');
	 end
         numdelays = delays(2)-delays(1)+1;
	else
	 numdelays = length(delays);
        end

        iprod = zeros(numdelays,1);
        [N,M] = size(swt);
        M = M-1;

        %pre-multiply pattern by 2^(-gamma*m) weighting
        pcoeff = pattern(:,3).*(2 .^(-gamma*pattern(:,1)));

        pindices = pattern(:,2);
        pscales =pattern(:,1);
  
        scaleind = find(pscales == M+1);
        if ~isempty(scaleind)
           pscales(scaleind) = zeros(length(scaleind),1);
        end


        if mode == 0
	       	delay = delays(1);
        	for delay_ind = 1:numdelays
 	          indices = modulo_add(pindices,N,delay);
        	  iprod(delay_ind) = sum(swt(N*pscales + indices).*pcoeff);
	          delay = delay+1;
	        end
        else
        	for delay_ind = 1:numdelays
 	          indices = modulo_add(pindices,N,delays(delay_ind));
        	  iprod(delay_ind) = sum(swt(N*pscales + indices).*pcoeff);
	        end
 	end




% Copyright (c) 1998 Thomas T. Liu

