function handles = plotroc(det_stat,d);
%
% handles = plotroc(det_stat,d);
%
% Inputs
%   det_stat: matrix of detection statistics
%          d: desired performance index, use [] for none
%
%  Outputs 
%   handles: handles to the plots

        pfatext = 'Probability of False Alarm';
        pdtext = 'Probability of Detection';
	numcurves = size(det_stat,2)/2;
	numtests = size(det_stat,1);
	M = 1000;
	pfa_pd = zeros(M,2*numcurves);

	for k = 1:numcurves
		coffset = 2*(k-1);
	 	fail_stat = det_stat(:,1+coffset);
		pass_stat = det_stat(:,2+coffset);

        	minstat = min(fail_stat);
  		maxstat = max(pass_stat);

		threshold = minstat:(maxstat-minstat)/M:maxstat;
		M = length(threshold);

		for m = 1:M
		 pfa_pd(m,1+coffset) = sum(fail_stat >= threshold(m))/numtests;
		 pfa_pd(m,2+coffset) = sum(pass_stat >= threshold(m))/numtests;
		end

	end
        handles = [];

	linetypes = ['r- ';'g-.';'b: ';'k--';'y- ';'m-.';'c: ';'r--';'g: '];
	numtypes = size(linetypes,1);
	for k = 1:numcurves
	  coffset = 2*(k-1);
	  h = plot(pfa_pd(:,1+coffset),pfa_pd(:,2+coffset),...
		linetypes(rem(k-1,numtypes)+1,:));
          handles = [handles;h];
	  hold on;
        end


        if ~isempty(d);	  
		beta = -3:.1:4;
		pfa = qfunc(beta);
		pd = qfunc(beta -d);
		plot(pfa,pd,'b-');
		hold off;
        end


        set(gca,'FontSize',11,'FontName','Times');
	xlabel(pfatext);
	ylabel(pdtext);
	hold off;


% Copyright (c) 1998 Thomas T. Liu

 